/*
 * Copyright(C) 2003  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 126987 $
 * \date $Date:: 2015-09-08 17:51:58 +0400#$
 * \author $Author: pav $
 *
 * \brief    ,    
 */

#ifndef __ENCODEAND_H
#define __ENCODEAND_H

#include "ASN1Util.h"
#include "WinCryptEx.h"

void ASN1SignatureToBlob (
    CACMPT_BLOB& signature_blob, const ASN1TDynBitStr &signature, BOOL fFormat=TRUE);

bool ASN1EncodeBuffer_EncodeAndVerifyUnsafe( 
    ASN1BEREncodeBuffer &enc_buffer, ASN1CType& obj,
    const ASN1TDynBitStr &signature, const ASN1T_SubjectPublicKeyInfo &key,
    HCRYPTPROV hProv, DWORD hash_calg );


/* This class is to fix the bug in ASN1C reference counting. When you
 * copy ASN1C_... class (generated by ASN1C compiler) like this:
 *
 * ASN1C_ResponderID objCopy(sourceObj);
 *
 * AND corresponding ASN1T_... class inherits ASN1TPDU:
 *
 * typedef struct EXTERN ASN1T_ResponderID : public ASN1TPDU { ... };
 * 
 * You get ASN1CTXT leak, because the reference counter for context is
 * incremented twice (in ASN1CType ctor and in ASN1C_...::getCopy), but
 * decremented only once (in ASN1CType dtor), because objCopt.msgData is
 * a reference, and no dtor is called for object allocated in ASN1C_...::getCopy.
 */

template <typename ASN1_T> inline void conditionalCtxtUnref( ASN1_T& /*t*/ ) { return; }

template <typename T>
class Asn1CtxtUnrefInDestructor : public T {
public:
    Asn1CtxtUnrefInDestructor( T& src ): T(src) { }
    ~Asn1CtxtUnrefInDestructor() { conditionalCtxtUnref(this->msgData); }
    static void conditionalCtxtUnref( ASN1TPDU& t ) { t.unsetContext(); }
};

template <class T>
bool ASN1EncodeBuffer_EncodeAndVerify(
    ASN1BEREncodeBuffer &enc_buffer, T& obj,
    const ASN1TDynBitStr &signature, const ASN1T_SubjectPublicKeyInfo &key,
    HCRYPTPROV hProv, DWORD hash_calg ) {

    // compensating missing context reference count decrement for objCopy.msgData
    Asn1CtxtUnrefInDestructor<T> objCopy(obj);
    return ASN1EncodeBuffer_EncodeAndVerifyUnsafe(enc_buffer,objCopy,
        signature,key,hProv,hash_calg);
}

ASN1TDynBitStr* ASN1EncodeBuffer_EncodeAndSignUnsafe (
    HCRYPTPROV hProv,
    DWORD dwKeySpec,
    ASN1BEREncodeBuffer &enc_buffer,
    ASN1CType& obj
);

template <class T>
ASN1TDynBitStr* ASN1EncodeBuffer_EncodeAndSign (
    HCRYPTPROV hProv,
    DWORD dwKeySpec,
    ASN1BEREncodeBuffer &enc_buffer,
    T& obj) {

    // compensating missing context reference count decrement for objCopy.msgData
    Asn1CtxtUnrefInDestructor<T> objCopy(obj);
    return ASN1EncodeBuffer_EncodeAndSignUnsafe(hProv,dwKeySpec,enc_buffer,objCopy);
}

bool ASN1EncodeBuffer_EncodeAndVerifyUnsafe( 
    ASN1BEREncodeBuffer &enc_buffer, ASN1CType& obj,
    const ASN1TDynBitStr &signature, const ASN1T_SubjectPublicKeyInfo &key,
    const ASN1T_AlgorithmIdentifier &alg );

template <class T>
bool ASN1EncodeBuffer_EncodeAndVerify( 
    ASN1BEREncodeBuffer &enc_buffer, T& obj,
    const ASN1TDynBitStr &signature, const ASN1T_SubjectPublicKeyInfo &key,
    const ASN1T_AlgorithmIdentifier &alg ) {

    // compensating missing context reference count decrement for objCopy.msgData
    Asn1CtxtUnrefInDestructor<T> objCopy(obj);
    return ASN1EncodeBuffer_EncodeAndVerifyUnsafe(enc_buffer,objCopy,signature,
        key,alg);
}

static inline DWORD StrType( DWORD type )
{
    return type&0x0000ffff;
}

#endif // __ENCODEAND_H
