/*
 * Copyright(C) 2003  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 126987 $
 * \date $Date:: 2015-09-08 17:51:58 +0400#$
 * \author $Author: pav $
 *
 * \brief PKIXCMP_Server class
 */

#ifndef PKIXCMP_SERVER_H
#define PKIXCMP_SERVER_H

#include <utility>

#define CPCSP_C_SOURCE 30
#include "PKIXCMP_Base.h"
#include "WinCryptEx.h"

class PKIXCMP_Server : public PKIXCMP_Base {
public:
    PKIXCMP_Server( const char *ini_path, WndProv *wnd_prov );
    PKIXCMP_Server( const char *szProvider=0, DWORD dwProvType = PROV_GOST_2001_DH );
    ~PKIXCMP_Server();

    //    .
    // dn [in] : RDN  .
    // szContainer [in/out] :   .
    //      , ,
    //     "\\\\.\\HDIMAGE\\"
    static PKIXCMP_Server *GenerateRoot( const char *ini_path,
	const wchar_t *dn, 
	const CACMPT_Extensions& Extensions,
	const wchar_t *caname,
        bool keyExportable,
	const char *szProvider,
	int dwProvType,
	const char *szContainer,
	const CACMPT_Period &cert_period,
	const CACMPT_Date &notbefore,
	const char *cdp,
	const CACMPT_Period &crl_period,
	WndProv *wnd_prov);
    
    //     
    // ann_msg [out] :     
    // ann_size [in/out] :  ann_msg
    // szContainer [in/out] :    .
    //      , ,
    //     "\\\\.\\HDIMAGE\\"

    int UpdateRoot( const char *ini_path,
	int dwProvType,
        const char *szContainer, bool keyExportable, CACMPT_BLOB &ann_msg,
	PKIXCMP_Message *info, WndProv *wnd_prov = 0,
	CACMPT_BLOB *new_cert = NULL);

    int ChangePassword( WndProv *wnd_prov );

    //  
    void GenerateCRL(CACMPT_BLOB &crl_msg, const CrlInfo *info);

    //       
    // Request   .
    void CertReqMessage_PreProcess(const CACMPT_BLOB &cr_msg, CertificateInfo *info);

    //       
    //    .   Self-Issued
    // 
    void CertReqCert_PreProcess(const CACMPT_BLOB &cert, CertificateInfo *info);

    //   (   ),
    //   (      
    // SOAP )
    // change_info ,      
    //   ,     .
    void CertReqMessage_Process(CertificateInfo *info, CACMPT_BLOB &cp_msg);

    void CertReqMessage_Reject(RequestInfo &request, CACMPT_BLOB &cp_msg, bool toSign);

    const CACMPT_BLOB& GetCert () const { return MYCert; }

    //     .
    void RevAnnContent_Process(CACMPT_BLOB &ra_msg, const RevAnnContentInfo *info);

    unsigned long GetCSP() const { return hProv; }

    //    cross .
    int CrossReqMessage_EncodeAndSign(const CertificateInfo *info, CACMPT_BLOB &cr_msg);

    int ErrorMsgInfo_EncodeAndSign( const ErrorMsgInfo *em_info,
	CACMPT_BLOB &em_msg)
    { return
	PKIXCMP_Base::ErrorMsgInfo_EncodeAndSign( em_info, em_msg, 
	hProv, AT_SIGNATURE, MYCert);
    }
    int CertAnnContent_EncodeAndSign(CertificateInfo *info, CACMPT_BLOB &ann_msg);

    //       Root
    void BuildChain( encoded_certificate_list &path )
    {
	CertChain_VerifyCertificate(GetCert(), path, ADD_STORE, ADD_STORE_LEN);
    }

    //      Body
    int GenMsgContent_EncodeAndSign(PKIXCMP_Message *info, CACMPT_BLOB &msg);

protected:
    void CertReqMessage_Process(CertificateInfo *info,
	CACMPT_BLOB &cp_msg, CACMPT_SERIAL szSerial /* in/out */);
    void GenerateRoot (const wchar_t *dn, 
	const CACMPT_Extensions& Extensions,
	const char * szProvider,
	int dwProvType,
	const char *szContainer,
        bool keyExportable,
	const CACMPT_Period &period,
	const CACMPT_Date &notbefore,
	const char *cdp,
        WndProv *wnd_prov);
    void CertReqMessage_PreProcess(const CACMPT_BLOB &cr_msg,
	CertificateInfo *info, int dummy );
    void CertReqCert_PreProcess(const CACMPT_BLOB &cert, CertificateInfo *info, int dummy);
    void GenerateCRL(CACMPT_BLOB &crl_msg, const CrlInfo *info, const CACMPT_Period &period);

    void *hProvParts;
    void init (const CACMPT_BLOB& cert, WndProv *wnd_prov );
    void release ();
    void init ( const char * szProvider, DWORD dwProvType );

    std::string ini_path;
    static HCRYPTPROV create_container_root( WndProv *wnd_prov, 
	const char *container, const char *provider, DWORD provtype,
        bool keyExportable, void* &parts, HCRYPTPROV hProvVer );
    static void read_container_root( HCRYPTPROV hProvCA,
	WndProv *wnd_prov, void *&parts );
    static void close_container_root( HCRYPTPROV hProvCA, void *&parts );

public:
    static const CertificateStore *ADD_STORE;
    static const int ADD_STORE_LEN;

    virtual const CertificateStore *Get_ADD_STORE () const { return ADD_STORE; }
    virtual int Get_ADD_STORE_LEN () const { return ADD_STORE_LEN; }
};
#endif // PKIXCMP_SERVER_H
