/*
* Copyright(C) 2000  
*
*    , 
*    .
*
*        ,
* ,    ,
*     ,
* ,      
*     
*      .
*/

/*!
* \file $RCSfile$
* \version $Revision: 133722 $
* \date $Date:: 2016-02-02 13:10:11 +0400#$
* \author $Author: sagafyin $
* \brief        fat12.
*/

#include "fat12prj.h" /* Project (READER/FAT12) include.
 include    (READER/FAT12). */

#ifdef UNIX
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

static TSupErr enum_strings(TFat12ConnectEnum **enum_ptr)
{
    TSupErr code=SUP_ERR_NO;

    int i;
#ifdef SOLARIS
    if(!volmgt_running())
	return SUP_ERR_CONNECT;
#else
    struct stat st;
#endif /* SOLARIS */

    *enum_ptr = malloc( sizeof( TFat12ConnectEnum ) );
    if( *enum_ptr == NULL )
	return SUP_ERR_MEMORY;
    (*enum_ptr)->drive_strings=(*enum_ptr)->cur=NULL;

    for(i=0;i<8;i++){
	char buf[20];
	TFat12ConnectEnumElem *el;

#ifdef SOLARIS
	sprintf(buf,"/dev/rdiskette%d",i);
	if(volmgt_inuse(buf))
#else /* SOLARIS */
	sprintf(buf,"/dev/fd%d",i);
	if(stat(buf, &st)!=ENOENT)
#endif /* SOLARIS */
	{
	    el=(TFat12ConnectEnumElem*)malloc(sizeof(TFat12ConnectEnumElem));
	    if(!el){
		code=SUP_ERR_MEMORY;
		goto exit;
	    }
	    el->next=NULL;
	    el->connect_str=(TCHAR*)malloc(2*sizeof(TCHAR)); /* floppy#+'\0' */
	    if(!el->connect_str){
		code=SUP_ERR_MEMORY;
		free(el);
		goto exit;
	    }
	    _stprintf(el->connect_str,"%d",i);
	    if((*enum_ptr)->cur){
		(*enum_ptr)->cur->next=el;
		(*enum_ptr)->cur=el;
	    } else
		(*enum_ptr)->drive_strings=(*enum_ptr)->cur=el;
	}
    }
exit:
    if(code){
	free(*enum_ptr); /* Correct freeing of what we've allocated previously
			 is needed */
	*enum_ptr = NULL;
    }
    return code;
}
#endif /* UNIX */

/*! 
* \ingroup fat12_fun_reader
* \brief       .
*
*      .
* \param context [in]  . 
* \param info [in/out]  #TReaderInfoConnectOpen
* \sa #TReaderInfoConnectOpen, #READER_FUN_CONNECT_NEXT, 
*  #TSupSysFunction, #fat12_info_connect_close, #fat12_info_connect_next,
* \return  
* \retval #SUP_ERR_NO    
*  
* \retval #SUP_ERR_PARAM   
* \retval #SUP_ERR_MEMORY   
* \retval " " \ref fat12_os_error. 
*/
#ifndef UNDER_CE
TSupErr fat12_group_enum_open(
    TSupSysContext *context, 
    TSupSysInfo *info )
{
    TSupSysInfoGroupEnumOpen *inf = (TSupSysInfoGroupEnumOpen*)info;
    TFat12ConnectEnum *enum_ptr;
#ifdef UNIX
    TSupErr code;
#else /* UNIX */
    DWORD length;
#endif /* UNIX */

    UNUSED( context );
    SUPSYS_PRE_INFO( info, TSupSysInfoGroupEnumOpen );
    inf->size_of = sizeof( TSupSysInfoGroupEnumOpen );
    inf->add_info.length = 0;
    inf->add_info.info = NULL;
#ifdef UNIX
    if((code=enum_strings(&enum_ptr)))
	return code;
    inf->name.length=8;
#else /* UNIX */
    enum_ptr = malloc( sizeof( TFat12ConnectEnum ) );
    if( enum_ptr == NULL )
	return SUP_ERR_MEMORY;
    length = GetLogicalDriveStrings( 0, NULL );
    if( length < 40 ) /* Windows 95   . */
	length = 40;
    enum_ptr->drive_strings = malloc( ( length + 1 ) * sizeof( TCHAR ) );
    if( enum_ptr->drive_strings == NULL )
    {
	free( enum_ptr );
	return SUP_ERR_MEMORY;
    }
    enum_ptr->cur = enum_ptr->drive_strings;
    GetLogicalDriveStrings( length, enum_ptr->drive_strings );
    /*   . */
    inf->name.length = 0;
    while( *( enum_ptr->cur ) )
    {
	size_t l = _tcslen( enum_ptr->cur );
	if (isCPdisk(enum_ptr->cur))
	{
	    if( inf->name.length < l )
		inf->name.length = l;
	}
	enum_ptr->cur += l + 1;
    }
    if( !inf->name.length )
    {
	free( enum_ptr->drive_strings );
	free( enum_ptr );
	return SUP_ERR_CONNECT;
    }
#endif /* UNIX */
    // TODO: improve
    inf->name.length += _tcslen(_TEXT("FAT12_"));
    enum_ptr->cur = enum_ptr->drive_strings;
    if (inf->compute_mark_func)
#ifdef _WIN32
	inf->compute_mark_func(enum_ptr->drive_strings, &inf->change_mark);
#else //_WIN32
	inf->change_mark = 0;
#endif //_WIN32

    inf->add_info.length = sizeof( TFat12ConnectEnum );
    inf->add_info.info = (unsigned char*)enum_ptr;

    return SUP_ERR_NO;
}
#else
TSupErr fat12_group_enum_open(
    TSupSysContext *context, 
    TSupSysInfo *info )
{
    TSupSysInfoGroupEnumOpen *inf = (TSupSysInfoGroupEnumOpen*)info;
    TFat12ConnectEnum *enum_ptr;
    TSupErr code = SUP_ERR_NO;
    // Name that matches all storage cards
    TCHAR *szFname = TEXT("\\*");

    UNUSED( context );
    SUPSYS_PRE_INFO( info, TSupSysInfoGroupEnumOpen );
    inf->size_of = sizeof( TSupSysInfoGroupEnumOpen );
    inf->add_info.length = 0;
    inf->add_info.info = NULL;

    enum_ptr=(TFat12ConnectEnum*)malloc(sizeof(TFat12ConnectEnum));
    if(!enum_ptr){
	code=SUP_ERR_MEMORY;
	goto exit;
    }

    // Ensure that Storage Card exists.
    enum_ptr->hSearch = FindFirstFile(szFname, &enum_ptr->fd);
    if (enum_ptr->hSearch == INVALID_HANDLE_VALUE)
	return SUP_ERR_NO;
    inf->add_info.length = sizeof(TFat12ConnectEnum);
    inf->add_info.info = (unsigned char *)enum_ptr;
    inf->name.length = SUPSYSE_NICKNAME_LENGTH + 1;
exit:
    if(code){
	free(enum_ptr); /* Correct freeing of what we've allocated previously
			is needed */
	return code;
    }
    return code;
}
#endif

#if !defined( UNIX )

BOOL isCPdisk(TCHAR * name)
{
	BOOL res;

	res = (GetDriveType(name) == DRIVE_REMOVABLE);
	if (!res)
	{
		TCHAR buf[251];
		UINT len = GetSystemDirectory(buf, 250);

		if (len > 250 || len == 0)
		{
			return FALSE;
		}

		res = (GetDriveType(name) == DRIVE_FIXED);
		if (res)
		{
			res = _tcsncmp(buf, name, _tcslen(name));
		}
	};
	return res;
}

#endif /*!defined( UNIX )*/
